@@ -53,9 +53,20 @@ class ForwardContentProcessor(ContentProcessor):
     #
     def process(self, content: Content, sender: ID, msg: InstantMessage) -> Optional[Content]:
         assert isinstance(content, ForwardContent), 'forward content error: %s' % content
-        r_msg = content.forward
         # call messenger to process it
-        return self.messenger.process_reliable(msg=r_msg)
+        r_msg = self.messenger.process_reliable(msg=content.forward)
+        # check response
+        if r_msg is not None:
+            # Over The Top
+            return ForwardContent.new(message=r_msg)
+        # else:
+        #     receiver = content.forward.envelope.receiver
+        #     text = 'Message forwarded: %s' % receiver
+        #     return ReceiptCommand.new(message=text)
+
+        # NOTICE: decrypt failed, not for you?
+        #         it means you are asked to re-pack and forward this message
+        return None
 
 
 # register
